#pragma once

#include <cal.h>
#include <cublas_v2.h>
#include <inttypes.h>
#include <stdio.h>

#define CUBLASMP_VER_MAJOR 0
#define CUBLASMP_VER_MINOR 1
#define CUBLASMP_VER_PATCH 0
#define CUBLASMP_VER_BUILD 0
#define CUBLASMP_VERSION (CUBLASMP_VER_MAJOR * 1000 + CUBLASMP_VER_MINOR * 100 + CUBLASMP_VER_PATCH)

#ifdef __cplusplus
extern "C"
{
#endif

typedef enum
{
    CUBLASMP_GRID_LAYOUT_COL_MAJOR,
    CUBLASMP_GRID_LAYOUT_ROW_MAJOR
} cublasMpGridLayout_t;

struct cublasMpHandle;
typedef cublasMpHandle* cublasMpHandle_t;

struct cublasMpGrid;
typedef cublasMpGrid* cublasMpGrid_t;

struct cublasMpMatrixDescriptor;
typedef cublasMpMatrixDescriptor* cublasMpMatrixDescriptor_t;

cublasStatus_t cublasMpCreate(cublasMpHandle_t* handle, cudaStream_t stream);

cublasStatus_t cublasMpDestroy(cublasMpHandle_t handle);

cublasStatus_t cublasMpGetVersion(cublasMpHandle_t handle, int* version);

cublasStatus_t cublasMpSetMathMode(cublasMpHandle_t handle, cublasMath_t mode);

cublasStatus_t cublasMpGetMathMode(cublasMpHandle_t handle, cublasMath_t* mode);

cublasStatus_t cublasMpGridCreate(
    cublasMpHandle_t handle,
    int64_t nprow,
    int64_t npcol,
    cublasMpGridLayout_t layout,
    cal_comm_t comm,
    cublasMpGrid_t* grid);

cublasStatus_t cublasMpGridDestroy(cublasMpHandle_t handle, cublasMpGrid_t grid);

cublasStatus_t cublasMpMatrixDescriptorCreate(
    cublasMpHandle_t handle,
    int64_t m,
    int64_t n,
    int64_t mb,
    int64_t nb,
    int64_t lld,
    cudaDataType_t type,
    cublasMpGrid_t grid,
    cublasMpMatrixDescriptor_t* desc);

cublasStatus_t cublasMpMatrixDescriptorDestroy(cublasMpHandle_t handle, cublasMpMatrixDescriptor_t desc);

cublasStatus_t cublasMpTrsm_bufferSize(
    cublasMpHandle_t handle,
    cublasSideMode_t side,
    cublasFillMode_t uplo,
    cublasOperation_t trans,
    cublasDiagType_t diag,
    int64_t m,
    int64_t n,
    const void* alpha,
    const void* a,
    int64_t ia,
    int64_t ja,
    cublasMpMatrixDescriptor_t descA,
    void* b,
    int64_t ib,
    int64_t jb,
    cublasMpMatrixDescriptor_t descB,
    cudaDataType computeType,
    size_t* workspaceSizeInBytesOnDevice,
    size_t* workspaceSizeInBytesOnHost);

cublasStatus_t cublasMpTrsm(
    cublasMpHandle_t handle,
    cublasSideMode_t side,
    cublasFillMode_t uplo,
    cublasOperation_t trans,
    cublasDiagType_t diag,
    int64_t m,
    int64_t n,
    const void* alpha,
    const void* a,
    int64_t ia,
    int64_t ja,
    cublasMpMatrixDescriptor_t descA,
    void* b,
    int64_t ib,
    int64_t jb,
    cublasMpMatrixDescriptor_t descB,
    cudaDataType computeType,
    void* d_work,
    size_t workspaceSizeInBytesOnDevice,
    void* h_work,
    size_t workspaceSizeInBytesOnHost);

cublasStatus_t cublasMpGemm_bufferSize(
    cublasMpHandle_t handle,
    cublasOperation_t transA,
    cublasOperation_t transB,
    int64_t m,
    int64_t n,
    int64_t k,
    const void* alpha,
    const void* a,
    int64_t ia,
    int64_t ja,
    cublasMpMatrixDescriptor_t descA,
    const void* b,
    int64_t ib,
    int64_t jb,
    cublasMpMatrixDescriptor_t descB,
    const void* beta,
    void* c,
    int64_t ic,
    int64_t jc,
    cublasMpMatrixDescriptor_t descC,
    cudaDataType computeType,
    size_t* workspaceSizeInBytesOnDevice,
    size_t* workspaceSizeInBytesOnHost);

cublasStatus_t cublasMpGemm(
    cublasMpHandle_t handle,
    cublasOperation_t transA,
    cublasOperation_t transB,
    int64_t m,
    int64_t n,
    int64_t k,
    const void* alpha,
    const void* a,
    int64_t ia,
    int64_t ja,
    cublasMpMatrixDescriptor_t descA,
    const void* b,
    int64_t ib,
    int64_t jb,
    cublasMpMatrixDescriptor_t descB,
    const void* beta,
    void* c,
    int64_t ic,
    int64_t jc,
    cublasMpMatrixDescriptor_t descC,
    cudaDataType computeType,
    void* d_work,
    size_t workspaceSizeInBytesOnDevice,
    void* h_work,
    size_t workspaceSizeInBytesOnHost);

cublasStatus_t cublasMpSyrk_bufferSize(
    cublasMpHandle_t handle,
    cublasFillMode_t uplo,
    cublasOperation_t trans,
    int64_t n,
    int64_t k,
    const void* alpha,
    const void* a,
    int64_t ia,
    int64_t ja,
    cublasMpMatrixDescriptor_t descA,
    const void* beta,
    void* c,
    int64_t ic,
    int64_t jc,
    cublasMpMatrixDescriptor_t descC,
    cudaDataType computeType,
    size_t* workspaceSizeInBytesOnDevice,
    size_t* workspaceSizeInBytesOnHost);

cublasStatus_t cublasMpSyrk(
    cublasMpHandle_t handle,
    cublasFillMode_t uplo,
    cublasOperation_t trans,
    int64_t n,
    int64_t k,
    const void* alpha,
    const void* a,
    int64_t ia,
    int64_t ja,
    cublasMpMatrixDescriptor_t descA,
    const void* beta,
    void* c,
    int64_t ic,
    int64_t jc,
    cublasMpMatrixDescriptor_t descC,
    cudaDataType computeType,
    void* d_work,
    size_t workspaceSizeInBytesOnDevice,
    void* h_work,
    size_t workspaceSizeInBytesOnHost);

int64_t cublasMpNumroc(int64_t n, int64_t nb, uint32_t iproc, uint32_t isrcproc, uint32_t nprocs);

typedef void (*cublasMpLoggerCallback_t)(int logLevel, const char* functionName, const char* message);

cublasStatus_t cublasMpLoggerSetCallback(cublasMpLoggerCallback_t callback);

cublasStatus_t cublasMpLoggerSetFile(FILE* file);

cublasStatus_t cublasMpLoggerOpenFile(const char* logFile);

cublasStatus_t cublasMpLoggerSetLevel(int level);

cublasStatus_t cublasMpLoggerSetMask(int mask);

cublasStatus_t cublasMpLoggerForceDisable();

#ifdef __cplusplus
}
#endif
